@@ -374,6 +374,10 @@ pub struct Grid {
     alternative_lines_above_viewport_and_cursor: Option<(VecDeque<Row>, Vec<Row>, Cursor)>,
     cursor: Cursor,
     saved_cursor_position: Option<Cursor>,
+    // FIXME: change scroll_region to be (usize, usize) - where the top line is always the first
+    // line of the viewport and the bottom line the last unless it's changed with CSI r and friends
+    // Having it as an Option causes lots of complexity and issues, and according to DECSTBM, this
+    // should be the behaviour anyway
     scroll_region: Option<(usize, usize)>,
     active_charset: CharsetIndex,
     preceding_char: Option<TerminalCharacter>,
@@ -452,16 +456,14 @@ impl Grid {
         self.output_buffer.update_line(line_index);
     }
     pub fn advance_to_next_tabstop(&mut self, styles: CharacterStyles) {
-        let mut next_tabstop = None;
-        for tabstop in self.horizontal_tabstops.iter() {
-            if *tabstop > self.cursor.x {
-                next_tabstop = Some(tabstop);
-                break;
-            }
-        }
+        let next_tabstop = self
+            .horizontal_tabstops
+            .iter()
+            .copied()
+            .find(|&tabstop| tabstop > self.cursor.x);
         match next_tabstop {
             Some(tabstop) => {
-                self.cursor.x = *tabstop;
+                self.cursor.x = tabstop;
             }
             None => {
                 self.cursor.x = self.width.saturating_sub(1);
@@ -473,16 +475,15 @@ impl Grid {
         self.output_buffer.update_line(self.cursor.y);
     }
     pub fn move_to_previous_tabstop(&mut self) {
-        let mut previous_tabstop = None;
-        for tabstop in self.horizontal_tabstops.iter() {
-            if *tabstop >= self.cursor.x {
-                break;
-            }
-            previous_tabstop = Some(tabstop);
-        }
+        let previous_tabstop = self
+            .horizontal_tabstops
+            .iter()
+            .rev()
+            .copied()
+            .find(|&tabstop| tabstop < self.cursor.x);
         match previous_tabstop {
             Some(tabstop) => {
-                self.cursor.x = *tabstop;
+                self.cursor.x = tabstop;
             }
             None => {
                 self.cursor.x = 0;
@@ -502,7 +503,7 @@ impl Grid {
 
     fn recalculate_scrollback_buffer_count(&self) -> usize {
         let mut scrollback_buffer_count = 0;
-        for row in self.lines_above.iter() {
+        for row in &self.lines_above {
             let row_width = row.width();
             // rows in lines_above are unwrapped, so we need to account for that
             if row_width > self.width {
@@ -527,7 +528,7 @@ impl Grid {
         self.saved_cursor_position = Some(self.cursor.clone());
     }
     fn restore_cursor_position(&mut self) {
-        if let Some(saved_cursor_position) = self.saved_cursor_position.as_ref() {
+        if let Some(saved_cursor_position) = &self.saved_cursor_position {
             self.cursor = saved_cursor_position.clone();
         }
     }
@@ -697,7 +698,7 @@ impl Grid {
 
             // trim lines after the last empty space that has no following character, because
             // terminals don't trim empty lines
-            for line in viewport_canonical_lines.iter_mut() {
+            for line in &mut viewport_canonical_lines {
                 let mut trim_at = None;
                 for (index, character) in line.columns.iter().enumerate() {
                     if character.character != EMPTY_TERMINAL_CHARACTER.character {
@@ -936,13 +937,30 @@ impl Grid {
                     // the state is corrupted
                     return;
                 }
-                self.viewport.remove(scroll_region_top);
-                let columns = VecDeque::from(vec![EMPTY_TERMINAL_CHARACTER; self.width]);
-                if self.viewport.len() >= scroll_region_bottom {
-                    self.viewport
-                        .insert(scroll_region_bottom, Row::from_columns(columns).canonical());
-                } else {
+                if scroll_region_bottom == self.height - 1 && scroll_region_top == 0 {
+                    let row_count_to_transfer = 1;
+                    let transferred_rows_count = transfer_rows_from_viewport_to_lines_above(
+                        &mut self.viewport,
+                        &mut self.lines_above,
+                        row_count_to_transfer,
+                        self.width,
+                    );
+                    self.scrollback_buffer_lines = subtract_isize_from_usize(
+                        self.scrollback_buffer_lines,
+                        transferred_rows_count,
+                    );
+                    let columns = VecDeque::from(vec![EMPTY_TERMINAL_CHARACTER; self.width]);
                     self.viewport.push(Row::from_columns(columns).canonical());
+                    self.selection.move_up(1);
+                } else {
+                    self.viewport.remove(scroll_region_top);
+                    let columns = VecDeque::from(vec![EMPTY_TERMINAL_CHARACTER; self.width]);
+                    if self.viewport.len() >= scroll_region_bottom {
+                        self.viewport
+                            .insert(scroll_region_bottom, Row::from_columns(columns).canonical());
+                    } else {
+                        self.viewport.push(Row::from_columns(columns).canonical());
+                    }
                 }
                 self.output_buffer.update_all_lines(); // TODO: only update scroll region lines
                 return;
@@ -1113,7 +1131,7 @@ impl Grid {
     pub fn clear_all(&mut self, replace_with: TerminalCharacter) {
         let replace_with_columns = VecDeque::from(vec![replace_with; self.width]);
         self.replace_characters_in_line_after_cursor(replace_with);
-        for row in self.viewport.iter_mut() {
+        for row in &mut self.viewport {
             row.replace_columns(replace_with_columns.clone());
         }
         self.output_buffer.update_all_lines();
@@ -1226,6 +1244,7 @@ impl Grid {
     pub fn set_scroll_region(&mut self, top_line_index: usize, bottom_line_index: Option<usize>) {
         let bottom_line_index = bottom_line_index.unwrap_or(self.height);
         self.scroll_region = Some((top_line_index, bottom_line_index));
+        self.move_cursor_to(0, 0, EMPTY_TERMINAL_CHARACTER); // DECSTBM moves the cursor to column 1 line 1 of the page
     }
     pub fn clear_scroll_region(&mut self) {
         self.scroll_region = None;
@@ -1459,7 +1478,7 @@ impl Grid {
         if self.title_stack.len() > MAX_TITLE_STACK_SIZE {
             self.title_stack.remove(0);
         }
-        if let Some(title) = self.title.as_ref() {
+        if let Some(title) = &self.title {
             self.title_stack.push(title.clone());
         }
     }
@@ -1771,7 +1790,7 @@ impl Perform for Grid {
                             alternative_lines_above,
                             alternative_viewport,
                             alternative_cursor,
-                        )) = self.alternative_lines_above_viewport_and_cursor.as_mut()
+                        )) = &mut self.alternative_lines_above_viewport_and_cursor
                         {
                             std::mem::swap(&mut self.lines_above, alternative_lines_above);
                             std::mem::swap(&mut self.viewport, alternative_viewport);
@@ -2137,7 +2156,7 @@ impl Row {
             Row::new(width)
         } else {
             let mut first_row = rows.remove(0);
-            for row in rows.iter_mut() {
+            for row in &mut rows {
                 first_row.append(&mut row.columns);
             }
             first_row
@@ -2153,14 +2172,14 @@ impl Row {
     }
     pub fn width(&self) -> usize {
         let mut width = 0;
-        for terminal_character in self.columns.iter() {
+        for terminal_character in &self.columns {
             width += terminal_character.width;
         }
         width
     }
     pub fn excess_width(&self) -> usize {
         let mut acc = 0;
-        for terminal_character in self.columns.iter() {
+        for terminal_character in &self.columns {
             if terminal_character.width > 1 {
                 acc += terminal_character.width - 1;
             }
