@@ -11,18 +11,24 @@ pub enum TerminalAction {
 
 #[derive(Clone, Debug, Deserialize, Default, Serialize, PartialEq, Eq)]
 pub struct RunCommand {
+    #[serde(alias = "cmd")]
     pub command: PathBuf,
     #[serde(default)]
     pub args: Vec<String>,
+    #[serde(default)]
+    pub cwd: Option<PathBuf>,
 }
 
 /// Intermediate representation
 #[derive(Clone, Debug, Deserialize, Default, Serialize, PartialEq, Eq)]
 pub struct RunCommandAction {
+    #[serde(rename = "cmd")]
     pub command: PathBuf,
     #[serde(default)]
     pub args: Vec<String>,
     #[serde(default)]
+    pub cwd: Option<PathBuf>,
+    #[serde(default)]
     pub direction: Option<Direction>,
 }
 
@@ -31,6 +37,7 @@ impl From<RunCommandAction> for RunCommand {
         RunCommand {
             command: action.command,
             args: action.args,
+            cwd: action.cwd,
         }
     }
 }
