@@ -133,6 +133,8 @@ impl fmt::Display for RunPluginLocation {
 pub struct Layout {
     pub direction: Direction,
     #[serde(default)]
+    pub pane_name: Option<String>,
+    #[serde(default)]
     pub parts: Vec<Layout>,
     pub split_size: Option<SplitSize>,
     pub run: Option<Run>,
@@ -162,7 +164,7 @@ pub struct LayoutFromYamlIntermediate {
 
 // The struct that is used to deserialize the layout from
 // a yaml configuration file
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Default)]
 #[serde(crate = "self::serde")]
 #[serde(default)]
 pub struct LayoutFromYaml {
@@ -254,8 +256,8 @@ impl LayoutFromYamlIntermediate {
     ) -> LayoutFromYamlIntermediateResult {
         match layout_dir {
             Some(dir) => Self::from_path(&dir.join(layout))
-                .or_else(|_| LayoutFromYamlIntermediate::from_default_assets(layout.as_path())),
-            None => LayoutFromYamlIntermediate::from_default_assets(layout.as_path()),
+                .or_else(|_| LayoutFromYamlIntermediate::from_default_assets(layout)),
+            None => LayoutFromYamlIntermediate::from_default_assets(layout),
         }
     }
     // Currently still needed but on nightly
@@ -277,19 +279,19 @@ impl LayoutFromYamlIntermediate {
     // once serde-yaml supports zero-copy
     pub fn default_from_assets() -> LayoutFromYamlIntermediateResult {
         let layout: LayoutFromYamlIntermediate =
-            serde_yaml::from_str(String::from_utf8(setup::DEFAULT_LAYOUT.to_vec())?.as_str())?;
+            serde_yaml::from_str(&String::from_utf8(setup::DEFAULT_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
 
     pub fn strider_from_assets() -> LayoutFromYamlIntermediateResult {
         let layout: LayoutFromYamlIntermediate =
-            serde_yaml::from_str(String::from_utf8(setup::STRIDER_LAYOUT.to_vec())?.as_str())?;
+            serde_yaml::from_str(&String::from_utf8(setup::STRIDER_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
 
     pub fn disable_status_from_assets() -> LayoutFromYamlIntermediateResult {
         let layout: LayoutFromYamlIntermediate =
-            serde_yaml::from_str(String::from_utf8(setup::NO_STATUS_LAYOUT.to_vec())?.as_str())?;
+            serde_yaml::from_str(&String::from_utf8(setup::NO_STATUS_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
 }
@@ -331,9 +333,10 @@ impl LayoutFromYaml {
     #[allow(clippy::ptr_arg)]
     pub fn from_dir(layout: &PathBuf, layout_dir: Option<&PathBuf>) -> LayoutFromYamlResult {
         match layout_dir {
-            Some(dir) => Self::new(&dir.join(layout))
-                .or_else(|_| Self::from_default_assets(layout.as_path())),
-            None => Self::from_default_assets(layout.as_path()),
+            Some(dir) => {
+                Self::new(&dir.join(layout)).or_else(|_| Self::from_default_assets(layout))
+            }
+            None => Self::from_default_assets(layout),
         }
     }
 
@@ -372,19 +375,19 @@ impl LayoutFromYaml {
     // once serde-yaml supports zero-copy
     pub fn default_from_assets() -> LayoutFromYamlResult {
         let layout: LayoutFromYaml =
-            serde_yaml::from_str(String::from_utf8(setup::DEFAULT_LAYOUT.to_vec())?.as_str())?;
+            serde_yaml::from_str(&String::from_utf8(setup::DEFAULT_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
 
     pub fn strider_from_assets() -> LayoutFromYamlResult {
         let layout: LayoutFromYaml =
-            serde_yaml::from_str(String::from_utf8(setup::STRIDER_LAYOUT.to_vec())?.as_str())?;
+            serde_yaml::from_str(&String::from_utf8(setup::STRIDER_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
 
     pub fn disable_status_from_assets() -> LayoutFromYamlResult {
         let layout: LayoutFromYaml =
-            serde_yaml::from_str(String::from_utf8(setup::NO_STATUS_LAYOUT.to_vec())?.as_str())?;
+            serde_yaml::from_str(&String::from_utf8(setup::NO_STATUS_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
 }
@@ -410,6 +413,8 @@ fn default_as_some_true() -> Option<bool> {
 pub struct LayoutTemplate {
     pub direction: Direction,
     #[serde(default)]
+    pub pane_name: Option<String>,
+    #[serde(default)]
     pub borderless: bool,
     #[serde(default)]
     pub parts: Vec<LayoutTemplate>,
@@ -453,6 +458,7 @@ impl LayoutTemplate {
 pub struct TabLayout {
     #[serde(default)]
     pub direction: Direction,
+    pub pane_name: Option<String>,
     #[serde(default)]
     pub borderless: bool,
     #[serde(default)]
@@ -465,7 +471,7 @@ pub struct TabLayout {
 
 impl TabLayout {
     fn check(&self) -> Result<TabLayout, ConfigError> {
-        for part in self.parts.iter() {
+        for part in &self.parts {
             part.check()?;
             if !part.name.is_empty() {
                 return Err(ConfigError::LayoutNameInTab(LayoutNameInTabError));
@@ -479,7 +485,7 @@ impl Layout {
     pub fn total_terminal_panes(&self) -> usize {
         let mut total_panes = 0;
         total_panes += self.parts.len();
-        for part in self.parts.iter() {
+        for part in &self.parts {
             match part.run {
                 Some(Run::Command(_)) | None => {
                     total_panes += part.total_terminal_panes();
@@ -493,7 +499,7 @@ impl Layout {
     pub fn total_borderless_panes(&self) -> usize {
         let mut total_borderless_panes = 0;
         total_borderless_panes += self.parts.iter().filter(|p| p.borderless).count();
-        for part in self.parts.iter() {
+        for part in &self.parts {
             total_borderless_panes += part.total_borderless_panes();
         }
         total_borderless_panes
@@ -503,7 +509,7 @@ impl Layout {
         if self.parts.is_empty() {
             run_instructions.push(self.run.clone());
         }
-        for part in self.parts.iter() {
+        for part in &self.parts {
             let mut current_runnables = part.extract_run_instructions();
             run_instructions.append(&mut current_runnables);
         }
@@ -549,7 +555,7 @@ fn layout_size(direction: Direction, layout: &Layout) -> usize {
                 .parts
                 .iter()
                 .map(|p| child_layout_size(direction, layout.direction, p))
-                .sum::<usize>();
+                .sum();
             max(size, children_size)
         }
     }
@@ -702,6 +708,7 @@ impl TryFrom<TabLayout> for Layout {
     fn try_from(tab: TabLayout) -> Result<Self, Self::Error> {
         Ok(Layout {
             direction: tab.direction,
+            pane_name: tab.pane_name,
             borderless: tab.borderless,
             parts: Self::from_vec_tab_layout(tab.parts)?,
             split_size: tab.split_size,
@@ -714,6 +721,7 @@ impl From<TabLayout> for LayoutTemplate {
     fn from(tab: TabLayout) -> Self {
         Self {
             direction: tab.direction,
+            pane_name: tab.pane_name,
             borderless: tab.borderless,
             parts: Self::from_vec_tab_layout(tab.parts),
             body: false,
@@ -729,6 +737,7 @@ impl TryFrom<LayoutTemplate> for Layout {
     fn try_from(template: LayoutTemplate) -> Result<Self, Self::Error> {
         Ok(Layout {
             direction: template.direction,
+            pane_name: template.pane_name,
             borderless: template.borderless,
             parts: Self::from_vec_template_layout(template.parts)?,
             split_size: template.split_size,
@@ -751,6 +760,7 @@ impl Default for TabLayout {
             split_size: None,
             run: None,
             name: String::new(),
+            pane_name: None,
         }
     }
 }
@@ -759,10 +769,12 @@ impl Default for LayoutTemplate {
     fn default() -> Self {
         Self {
             direction: Direction::Horizontal,
+            pane_name: None,
             body: false,
             borderless: false,
             parts: vec![LayoutTemplate {
                 direction: Direction::Horizontal,
+                pane_name: None,
                 body: true,
                 borderless: false,
                 split_size: None,
@@ -775,17 +787,6 @@ impl Default for LayoutTemplate {
     }
 }
 
-impl Default for LayoutFromYaml {
-    fn default() -> Self {
-        Self {
-            session: SessionFromYaml::default(),
-            template: LayoutTemplate::default(),
-            borderless: false,
-            tabs: vec![],
-        }
-    }
-}
-
 impl Default for Direction {
     fn default() -> Self {
         Direction::Horizontal
