@@ -40,10 +40,12 @@ fn default_layout_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Vertical,
                 borderless: true,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(1)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -54,6 +56,7 @@ fn default_layout_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: None,
                 run: None,
@@ -61,6 +64,7 @@ fn default_layout_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: true,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(2)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -84,10 +88,12 @@ fn default_layout_new_tab_correct() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Vertical,
                 borderless: true,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(1)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -98,6 +104,7 @@ fn default_layout_new_tab_correct() {
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: None,
                 run: None,
@@ -105,6 +112,7 @@ fn default_layout_new_tab_correct() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: true,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(2)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -168,13 +176,16 @@ fn three_panes_with_tab_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![Layout {
             direction: Direction::Vertical,
             borderless: false,
+            pane_name: None,
             parts: vec![
                 Layout {
                     direction: Direction::Horizontal,
                     borderless: false,
+                    pane_name: None,
                     parts: vec![],
                     split_size: Some(SplitSize::Percent(50.0)),
                     run: None,
@@ -182,10 +193,12 @@ fn three_panes_with_tab_merged_correctly() {
                 Layout {
                     direction: Direction::Horizontal,
                     borderless: false,
+                    pane_name: None,
                     parts: vec![
                         Layout {
                             direction: Direction::Vertical,
                             borderless: false,
+                            pane_name: None,
                             parts: vec![],
                             split_size: Some(SplitSize::Percent(50.0)),
                             run: None,
@@ -193,6 +206,7 @@ fn three_panes_with_tab_merged_correctly() {
                         Layout {
                             direction: Direction::Vertical,
                             borderless: false,
+                            pane_name: None,
                             parts: vec![],
                             split_size: Some(SplitSize::Percent(50.0)),
                             run: None,
@@ -220,9 +234,11 @@ fn three_panes_with_tab_new_tab_is_correct() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![Layout {
             direction: Direction::Horizontal,
             borderless: false,
+            pane_name: None,
             parts: vec![],
             split_size: None,
             run: None,
@@ -260,10 +276,12 @@ fn three_panes_with_tab_and_default_plugins_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(1)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -274,10 +292,12 @@ fn three_panes_with_tab_and_default_plugins_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![
                     Layout {
                         direction: Direction::Horizontal,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![],
                         split_size: Some(SplitSize::Percent(50.0)),
                         run: None,
@@ -285,10 +305,12 @@ fn three_panes_with_tab_and_default_plugins_merged_correctly() {
                     Layout {
                         direction: Direction::Horizontal,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![
                             Layout {
                                 direction: Direction::Vertical,
                                 borderless: false,
+                                pane_name: None,
                                 parts: vec![],
                                 split_size: Some(SplitSize::Percent(50.0)),
                                 run: None,
@@ -296,6 +318,7 @@ fn three_panes_with_tab_and_default_plugins_merged_correctly() {
                             Layout {
                                 direction: Direction::Vertical,
                                 borderless: false,
+                                pane_name: None,
                                 parts: vec![],
                                 split_size: Some(SplitSize::Percent(50.0)),
                                 run: None,
@@ -311,6 +334,7 @@ fn three_panes_with_tab_and_default_plugins_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(2)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -334,10 +358,12 @@ fn three_panes_with_tab_and_default_plugins_new_tab_is_correct() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(1)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -348,6 +374,7 @@ fn three_panes_with_tab_and_default_plugins_new_tab_is_correct() {
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: None,
                 run: None,
@@ -355,6 +382,7 @@ fn three_panes_with_tab_and_default_plugins_new_tab_is_correct() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Fixed(2)),
                 run: Some(Run::Plugin(RunPlugin {
@@ -396,14 +424,17 @@ fn deeply_nested_tab_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![
                     Layout {
                         direction: Direction::Horizontal,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![],
                         split_size: Some(SplitSize::Percent(21.0)),
                         run: None,
@@ -411,10 +442,12 @@ fn deeply_nested_tab_merged_correctly() {
                     Layout {
                         direction: Direction::Vertical,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![
                             Layout {
                                 direction: Direction::Horizontal,
                                 borderless: false,
+                                pane_name: None,
                                 parts: vec![],
                                 split_size: Some(SplitSize::Percent(22.0)),
                                 run: None,
@@ -422,10 +455,12 @@ fn deeply_nested_tab_merged_correctly() {
                             Layout {
                                 direction: Direction::Horizontal,
                                 borderless: false,
+                                pane_name: None,
                                 parts: vec![
                                     Layout {
                                         direction: Direction::Horizontal,
                                         borderless: false,
+                                        pane_name: None,
                                         parts: vec![],
                                         split_size: Some(SplitSize::Percent(23.0)),
                                         run: None,
@@ -433,6 +468,7 @@ fn deeply_nested_tab_merged_correctly() {
                                     Layout {
                                         direction: Direction::Horizontal,
                                         borderless: false,
+                                        pane_name: None,
                                         parts: vec![],
                                         split_size: Some(SplitSize::Percent(24.0)),
                                         run: None,
@@ -452,6 +488,7 @@ fn deeply_nested_tab_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Percent(15.0)),
                 run: None,
@@ -459,6 +496,7 @@ fn deeply_nested_tab_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Percent(15.0)),
                 run: None,
@@ -466,6 +504,7 @@ fn deeply_nested_tab_merged_correctly() {
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Percent(15.0)),
                 run: None,
@@ -504,10 +543,12 @@ fn three_tabs_tab_one_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Vertical,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: Some(SplitSize::Percent(50.0)),
                 run: None,
@@ -515,6 +556,7 @@ fn three_tabs_tab_one_merged_correctly() {
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: None,
                 run: None,
@@ -539,14 +581,17 @@ fn three_tabs_tab_two_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Vertical,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![
                     Layout {
                         direction: Direction::Horizontal,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![],
                         split_size: Some(SplitSize::Percent(50.0)),
                         run: None,
@@ -554,6 +599,7 @@ fn three_tabs_tab_two_merged_correctly() {
                     Layout {
                         direction: Direction::Horizontal,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![],
                         split_size: None,
                         run: None,
@@ -565,6 +611,7 @@ fn three_tabs_tab_two_merged_correctly() {
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: None,
                 run: None,
@@ -589,14 +636,17 @@ fn three_tabs_tab_three_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Vertical,
         borderless: false,
+        pane_name: None,
         parts: vec![
             Layout {
                 direction: Direction::Vertical,
                 borderless: false,
+                pane_name: None,
                 parts: vec![
                     Layout {
                         direction: Direction::Vertical,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![],
                         split_size: Some(SplitSize::Percent(50.0)),
                         run: None,
@@ -604,6 +654,7 @@ fn three_tabs_tab_three_merged_correctly() {
                     Layout {
                         direction: Direction::Horizontal,
                         borderless: false,
+                        pane_name: None,
                         parts: vec![],
                         split_size: None,
                         run: None,
@@ -615,6 +666,7 @@ fn three_tabs_tab_three_merged_correctly() {
             Layout {
                 direction: Direction::Horizontal,
                 borderless: false,
+                pane_name: None,
                 parts: vec![],
                 split_size: None,
                 run: None,
@@ -650,9 +702,11 @@ fn no_tabs_merged_correctly() {
     let merged_layout = Layout {
         direction: Direction::Horizontal,
         borderless: false,
+        pane_name: None,
         parts: vec![Layout {
             direction: Direction::Horizontal,
             borderless: false,
+            pane_name: None,
             parts: vec![],
             split_size: None,
             run: None,
@@ -699,6 +753,7 @@ fn no_layout_template_merged_correctly() {
                     split_size: None,
                     run: None,
                     borderless: false,
+                    pane_name: None,
                 },
                 Layout {
                     direction: Direction::Horizontal,
@@ -706,16 +761,41 @@ fn no_layout_template_merged_correctly() {
                     split_size: None,
                     run: None,
                     borderless: false,
+                    pane_name: None,
                 },
             ],
             split_size: None,
             run: None,
             borderless: false,
+            pane_name: None,
         }],
         split_size: None,
         run: None,
         borderless: false,
+        pane_name: None,
     };
 
     assert_eq!(merged_layout, tab_layout.try_into().unwrap());
 }
+
+#[test]
+fn session_name_to_layout_is_ok() {
+    let path = layout_test_dir("session-name-to-layout.yaml".into());
+    let layout_from_yaml = LayoutFromYaml::new(&path);
+    assert!(layout_from_yaml.is_ok());
+}
+
+#[test]
+fn session_name_to_layout_has_name() {
+    let path = layout_test_dir("session-name-to-layout.yaml".into());
+    let layout_from_yaml = LayoutFromYaml::new(&path);
+    let layout_template = layout_from_yaml.unwrap();
+    let session_layout = layout_template.session;
+
+    let expected_session = SessionFromYaml {
+        name: Some(String::from("zellij-session")),
+        attach: Some(true),
+    };
+
+    assert_eq!(expected_session, session_layout);
+}
